package com.biganto.visual.roompark.presentation.screen.auth

import com.biganto.visual.roompark.conductor.BigantoBasePresenter
import com.biganto.visual.roompark.domain.interactor.AuthInteractor
import com.biganto.visual.roompark.util.monades.ExceptionString
import io.reactivex.Observable
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import timber.log.Timber
import javax.inject.Inject

/**
 * Created by Vladislav Bogdashkin on 30.09.2019.
 */


class AuthScreenPresenter @Inject constructor(
    private val interactor: AuthInteractor
)
    : BigantoBasePresenter<AuthScreen, AuthScreenViewState>() {

    override fun attachView(view: AuthScreen) {
        super.attachView(view)
        Timber.d("on AttachView")
        restoreStateObservable.accept(AuthScreenViewState.Idle())
    }

    override fun detachView() {
        super.detachView()
        Timber.d("on DetachView")
        restoreStateObservable    }


    override fun defaultErrorViewStateHandler() =
        {e:ExceptionString -> AuthScreenViewState.SomeError(e)}

    override fun vsByCode(code: Int): (ExceptionString) -> AuthScreenViewState =
        when (code) {
            101 -> {e:ExceptionString -> AuthScreenViewState.SignInError(e)}
            111 -> {e:ExceptionString -> AuthScreenViewState.WrongLogin(e)}
            112 -> {e:ExceptionString -> AuthScreenViewState.WrongPassword(e)}
            else -> {e:ExceptionString -> AuthScreenViewState.SomeError(e)}
        }

    override fun bindIntents() {

        val onAuth = intent(AuthScreen::tryAuth)
            .flatMap <AuthScreenViewState> { model ->
                interactor.signIn(model.login, model.pwd)
                    .doOnNext { Timber.d("auth returned $it") }
                    .map { it }
                    .map<AuthScreenViewState> { AuthScreenViewState.SignedIn() }
                    .onErrorReturn{parseError(it)}
                    .startWith(Observable.just(AuthScreenViewState.Authorization()))
            }



        val state = restoreStateObservable
            .mergeWith(onAuth)
            .doOnError { Timber.e(it) }
                .onErrorReturn(::parseError)
            .subscribeOn(Schedulers.single())
            .observeOn(AndroidSchedulers.mainThread())

        subscribeViewState(state.cast(AuthScreenViewState::class.java), AuthScreen::render)
    }

}